<?php
/**
 * zogan - PHP template engine for XML
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Haruki Setoyama <haruki@planewave.org>
 * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @version 0.1- $Id: zogan.parser.php,v 1.1 2004/06/17 09:00:27 haruki Exp $
 * @link http://phpoot.sourceforge.jp/zogan/
 * @package zogan
 * @subpackage _builder
 */
/**
 * directory where ZOGAN library is in
 * @ignore
 */
if (!defined('ZOGAN_DIR')) {
    define('ZOGAN_DIR', dirname(__FILE__));
}
/**
 * Requires PEAR & XML_parser
 */
require_once 'PEAR.php';
require_once 'XML/Parser.php';
require_once 'zogan.handler.php';

/**
* parser with XML_Parser
* @package zogan
* @subpackage _builder
* @ignore
*/
class zogan_parser extends XML_Parser {

    var $_handler;

    function zogan_parser(&$zogan, $encoding = 'utf-8')
    {
        $vars = array_keys(get_class_vars('PEAR'));
        foreach ($vars as $var) {
            $this->$var =& $zogan->$var;
        }
        $this->_handler = new zogan_handler_xmlparser;
        $this->folding = false;
        XML_Parser::XML_Parser($encoding);
    }

    function build($template, $option)
    {
        $this->_handler->init($template, $option);

        $ret = $this->parseString($template);

        if (PEAR::isError($ret)) {
            return $ret;
        }
        return $this->_handler->getBuilt();
    }

    function startHandler($parser, $name, $attrs)
    {
        $this->_handler->startHandler($parser, $name, $attrs);
    }

    function endHandler($parser, $name)
    {
        $this->_handler->endHandler($parser, $name);
    }

    function cdataHandler($parser, $data)
    {
        $this->_handler->cdataHandler($parser, $data);
    }

    function defaultHandler($parser, $data)
    {
        if (preg_match('/^<!(.*)>$/sD', $data, $match)) {
            $this->_handler->escapeHandler($parser, $match[1]);
        }
        else {
            $this->_handler->defaultHandler($parser, $data);
        }
    }

    function piHandler($parser, $target, $data)
    {
        $this->_handler->piHandler($parser, $target, $data);
    }
}

/**
* handler for XML_Parser
* @package zogan
* @subpackage _builder
*/
class zogan_handler_xmlparser extends zogan_handler {
    function endHandler($parser, $close_tag)
    {
        $empty_tag = false;
        $pos = xml_get_current_byte_index($parser);
        if (substr($this->_template, $pos-2, 2) == '/>') {
            $empty_tag = true;
        }
        $this->_elements[$this->_level]->close($empty_tag);
        $this->_level--;
    }

    function cdataHandler($parser, $data)
    {
        if ($data == "\r\n" || $data == "\n" || $data == "\r") {
            $this->_elements[$this->_level]->child[]
                = new zogan_content_linefeed($data);
        }
        elseif ($data != '') {
            $this->_elements[$this->_level]->child[]
                = new zogan_content_cdata($data);
        }
    }

    function defaultHandler($parser, $data)
    {
        $this->_elements[$this->_level]->child[]
               = new zogan_content_default($data);
    }

    function _get_current_position(&$parser)
    {
        return xml_get_current_byte_index($parser);
    }
}

?>
